import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;

/**
 * Produces input function u(t) from mouse dragging.
 Copyright The University of British Columbia, Canada
 @author Dninesh K. Pai (pai@cs.ubc.ca)
 @author Kees van den Doel (kvdoel@cs.ubc.ca)
 */
public class DemoMouseScrape 
    implements MouseMotionListener, ComponentListener
{
    public static final int DIM = 2;

    /** component over which mouse events are gathered */
    Component component;

    /** current mouse position */
    int x = 0, y = 0;
  
    /** dimension of component, in pixels. Changed if component is resized */
    Dimension size; 

    /** true if input is scaled to [0,1] */
    boolean scaled =  true;

    /** Contruct input source, which produces mouse (x,y) positions
     * from dragging on <code>component</code>. 
     *
     * @param component Active component on which to drag
     * @param scaled if true, input is scaled to [0,1] in the component
     */
    DemoMouseScrape( Component component, boolean scaled) {
        this.component = component;
        this.scaled = scaled;
        size = component.getSize();
    }

    /** Initialize resources and start capture.
        Must call {@link #read} at <code>inputRate</code> to consume
        input data.
      
        @return true if successfully started.
    */
    public boolean start() {
        component.addMouseMotionListener(this);
        if (scaled) {
            component.addComponentListener(this);
        }
        return true;

    }

    /** Stop collecting data and free resources */
    public void stop() {
        component.removeMouseMotionListener(this);
        if (scaled) {
            component.removeComponentListener(this);
        }
    }

    /** read next input. A client of this class should arrange to
        read the data out at the inputRate. 
        Returns the latest mouse position.  */
    public void read(float[] u) {
        if (scaled) {
            u[0]  = (float) x / size.width ;
            u[1]  = (float) y / size.height ;
        }
        else {
            u[0]  = x;
            u[1]  = y;
        }
    }

    /** return raw mouse position, even if scaled values
        are being returned by read. Note, this is the current
        value and may be different than what is returned by read */
    public void readRaw(float[] rawU) {
        rawU[0]  = x;
        rawU[1]  = y;
    }


    /** return input dimension */
    public int getDimension() {
        return DIM;
    }

    ////////////////////////////////////////
    ///Implement ComponentListener
    public void componentResized(ComponentEvent e) {
        size = component.getSize();
    }

    public void componentHidden(ComponentEvent e) {};
    public void componentMoved(ComponentEvent e) {};
    public void componentShown(ComponentEvent e) {};

    ////////////////////////////////////////
    ///implement MouseMotionListener

    /** save current mouse position */
    public void mouseDragged(MouseEvent e) {
        x = e.getX();
        y = e.getY();
    }

    public void mouseMoved(MouseEvent e) {
    }

    static ModalSonicObject sob;
    static LoopForce af;
    static float vvx=0;
    static float vvy=0;
    static float vvv=0;
    
    public static void main (String[] args) {

        float srate = 44100;
        int bufferSize = 128;
        int bufferSizeJavaSound = 8*1024;
        if(args.length != 3) {
            System.out.println("Usage: java DemoMouseScrape 30 sword2.sy grid.wav");
            return;
        }
        try {
            sob = new ModalSonicObject(new ModalModel(args[1]),
                                       srate,bufferSize,bufferSizeJavaSound);
        } catch (java.io.FileNotFoundException ee) {
            System.out.println("Modes file not found\n");
        }
        af = new LoopForce(args[2]);
        sob.setAudioForce(af);
        sob.start();
            
        final int rate = Integer.parseInt(args[0]); // Hz
        //Create the top-level container and add viewer
        JFrame frame = new JFrame("DemoMouseScrape Test");
        //Finish setting up the frame, and show it.
        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
        frame.setSize(new Dimension(200,200));
        frame.setVisible(true);
        final DemoMouseScrape mi = new DemoMouseScrape(frame, true);
        Timer timer = new Timer();
        TimerTask task = new TimerTask() {                
                float[] u = new float[mi.getDimension()];
                int ibegin=0,iend=0; // begin and end pointers in ubuf
                float[] xbuf;
                float[] ybuf;
                final float[] bRamez5 = {
                    0.4436f,
                    0.2839f,
                    0.2225f,
                    -0.2225f,
                    -0.2839f,
                    -0.4436f
                };
                float[] bRamez;

                void insert(float x, float y) {
                    int newend = iend+1;
                    if(newend>=bRamez.length) {
                        newend = 0;
                    }
                    if(newend == ibegin) { // was full
                        ibegin++;
                        if(ibegin>=bRamez.length) {
                            ibegin = 0;
                        }
                        iend = newend;
                    } else { // was not full
                        iend = newend;
                    }
                    xbuf[iend] = x;
                    ybuf[iend] = y;
                }

                float firDiff(float[] x) {
                    int j=0;
                    float ret=0;
                    if(ibegin<iend) {
                        for(int i=iend;i>=ibegin;i--) {
                            ret += bRamez[j]*x[i];
                            //System.out.println(j);
                            j++;
                        }
                    } else {
                        for(int i=iend;i>=0;i--) {
                            ret += bRamez[j]*x[i];
                            //System.out.println(j);
                            j++;
                        }
                        for(int i=bRamez.length-1;i>=ibegin;i--) {
                            ret += bRamez[j]*x[i];
                            //System.out.println(j);
                            j++;
                        }
                    }
                    return ret;
                }
                
                public void clearHist() {
                    for(int i=0;i<bRamez.length;i++) {
                        xbuf[i] = ybuf[i] = 1.f;
                    }
                }

                    { 
                        bRamez = bRamez5;
                        xbuf = new float[bRamez.length];
                        ybuf = new float[bRamez.length];
                        clearHist();
                    }
                
                public void run() {
                    mi.read(u);
                    insert(u[0],u[1]);
                    vvx = firDiff(xbuf);
                    vvy = firDiff(ybuf);
                    vvv = (float)(5*Math.sqrt(vvx*vvx+vvy*vvy));
                    //System.out.println(vvv);
                    af.setSpeed(vvv);
                }
            };
        mi.start();
        timer.scheduleAtFixedRate(task, 0, (long) (1000.0/rate));
    }


    
}


