/**
   A force model based on looping through a buffer, loaded from an audio
   file.
   Copyright The University of British Columbia, Canada
   @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class LoopForce implements AudioForce {
    /** Buffer to loop. */
    protected AudioFileBuffer afBuffer;

    /** Playback volume. */
    protected float volume = 1.f;

    /** Loop speed through buffer in seconds per second. */
    protected float speed = 1.f;

    /** Current fractional position [0 1] of pointer in buffer. */
    protected float x = 0;

    /** Current integer position of pointer in buffer. */
    protected int ix = 0;
    
    /**  Current  fractional speed [0  1]  of pointer  in  buffer  per sample. */
    private float dx;
    
    /**  Current  integer speed of pointer  in  buffer  per sample. */
    private int dix;          

    /** Construct loop force from named file.
        @param fn Audio file name.
    */
    public LoopForce(String fn) {
        afBuffer = new AudioFileBuffer(fn);
        setSpeed(speed);
    }

    /** Set force magnitude.
       @param val Volume.
     */
    public void setVolume(float val) {
        volume = val;
    }

    /** Set loopspeed.
        @param speed Loop speed., 1 corresponding to original recorded speed.
    */
    public void setSpeed(float speed) {
        this.speed = speed;
        dix = (int)speed;
        dx = speed - dix;
    }

    /** Get next sample value, interpolating in between sample points.
     */
    private float getNextSample() {
        ix += dix;
        x += dx;
        if(x > 1.f) {
            x -= 1.f;
            ix++;
        }
        while(ix >= afBuffer.bufsz) {
            ix -= afBuffer.bufsz;
        }
        int next_index;
        if(ix == afBuffer.bufsz-1) {
            next_index = 0;
        } else {
            next_index = ix+1;
        }
        float val = (1.f - dx) * afBuffer.buf[ix] + dx * afBuffer.buf[next_index];
        return (float)(val * volume);
    }

    /** Create force.
        @param output user provided buffer for force returned.
        @param nsamples number of samples written to output buffer.
    */
    public void getForce(float [] output, int nsamples) {
        for(int k=0;k<nsamples;k++) {
            output[k] = getNextSample();
        }
    }

}


