import java.io.*;

/** Vibration model of object, capable of playing sound.
    Copyright The University of British Columbia, Canada
    @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class ModalSonicObject extends SonicObject {
    /** Audio force used for rendering. */
    protected AudioForce audioForce = null;
    
    /** Modal data. */    
    public ModalModel modalModel;
    
    /** Current barycentric location points. */
    private int p1=0,p2=0,p3=0;

    /** Current barycentric coordinates of location. */
    private float b1=1,b2=0,b3=0;

    /** State of filters. */
    private float [] yt_1, yt_2; 
    
    /** The transfer function of a reson filter is H(z) = 1/(1-twoRCosTheta/z + R2/z*z). */
    private float[] R2;

    /** The transfer function of a reson filter is H(z) = 1/(1-twoRCosTheta/z + R2/z*z). */
    private float[] twoRCosTheta;

    /** Reson filter gain. */
    private float[] ampR; 

    /** Cached values. */
    private float[] c_i;

    /** Scratch buffer. */
    float[] scratchBuf;
  
    /** Scale dampings.
     @param d damping scale. */
    public void setDamping(float dscale) {
        modalModel.dscale = dscale;
        computeFilter();
    }

    /** Scale frequencies.
     @param fscale frequency scale. */
    public void setFrequencyScale(float fscale) {
        modalModel.fscale = fscale;
        computeFilter();
    }

    /** Create and initialize, but don't set any modal parameters.
        @param srate sampling rate in Hertz.
        @param nf number of modes.
        @param np number of locations.
        @param bufferSize Buffer size used for real-time rendering.
        @param bufferSizeJavaSound Buffer size used by JavaSound.
     */
    public ModalSonicObject(float srate,int nf,int np,int bufferSize,int bufferSizeJavaSound) {
        super(srate,bufferSize,bufferSizeJavaSound);
        modalModel = new ModalModel(nf,np);
        allocate(nf,np);
    }

    /** Create and initialize with provided modal data.
        @param m modal model to load.
        @param srate sampling rate in Hertz.
        @param bufferSize Buffer size used for real-time rendering.
        @param bufferSizeJavaSound Buffer size used by JavaSound.
     */
    public ModalSonicObject(ModalModel m,float srate,int bufferSize,int bufferSizeJavaSound) {
        super(srate,bufferSize,bufferSizeJavaSound);
        modalModel = m;
        allocate(modalModel.nf,modalModel.np);
        computeFilter();
    }

    /** Reduce number of modes used.
     @param nf number of modes to use.
    */
    public void setNf(int nf) {
        if(nf < modalModel.nf) {
            modalModel.nfUsed = nf;
        }
    }

    /** Allocate data.
        @param nf number of modes.
        @param np number of locations.
     */
    private void allocate(int nf,int np) {
        R2 = new float[nf];
        twoRCosTheta = new float[nf];
        ampR = new float[nf];
        yt_1 = new float[nf];
        yt_2 = new float[nf];
        c_i = new float[nf];
        scratchBuf = new float[bufferSize];
        clearHistory();
    }
    

    /** Compute the filter coefficients used for real-time rendering
        from the modal model parameters. 
     */
    public void computeFilter() {
        computeResonCoeff();
        computeLocation();
    }

    /** Compute the reson coefficients from the modal model parameters.
        Cache values used in {@link #setLocation}.
     */
    public void computeResonCoeff() {
        for(int i=0;i<modalModel.nf;i++) {
            float tmp_r = (float)(Math.exp(-modalModel.dscale*modalModel.d[i]/srate));
            R2[i] = tmp_r*tmp_r;
            twoRCosTheta[i] = (float)(2*Math.cos(2*Math.PI*modalModel.fscale*modalModel.f[i]/srate)*tmp_r);
            c_i[i] = (float)(Math.sin(2*Math.PI*modalModel.fscale*modalModel.f[i]/srate)*tmp_r);
        }        
    }

    /** Compute gains.
     */
    private void computeLocation() {
        for(int i=0;i<modalModel.nf;i++) {
            ampR[i] = modalModel.ascale *c_i[i] *
                (b1* modalModel.a[p1][i] + b2* modalModel.a[p2][i] + b3* modalModel.a[p3][i]);
        }   
    }
    
    /** Compute the gain coefficients  from the modal model parameters at point p,
        given inside triangle of point p1,p2,p3, with barycentric coordinated b1,b2,b3
        @param p1 location index 1.
        @param p2 location index 2.
        @param p3 location index 3.
        @param b1 barycentric coordinate 1.
        @param b2 barycentric coordinate 2.
        @param b3 barycentric coordinate 3.
     */
    public void setLocation(int p1, int p2, int p3, float b1, float b2, float b3) {
        this.p1 = p1;
        this.p2 = p2;
        this.p3 = p3;
        this.b1 = b1;
        this.b2 = b2;
        this.b3 = b3;
        computeLocation();
    }

    /** Set state to non-vibrating.
     */
    public void clearHistory() {
        maxSignal = 0;
        for(int i=0;i<modalModel.nf;i++) {
            yt_1[i] = yt_2[i] = 0;
        }
    }
    
    /** Return audio force, which will usually be "applied" to the eardrum.
        nsamples must be smaller than bufferSize.
        @param output user provided buffer for force returned.
        @param nsamples number of samples written to output buffer.
    */
    public void getForce(float [] output, int nsamples) {
        audioForce.getForce(scratchBuf, nsamples);
        computeSoundBuffer(output, scratchBuf, nsamples);
    }

    /** Set the audio force.
        @param af audioForce to use.
    */
    public void setAudioForce(AudioForce af) {
        audioForce = af;
    }
    

    /** Apply external force[] and compute response.
        @param output user provided output buffer.
        @param force input force.
        @param nsamples number of samples to compute.
    */
    private void computeSoundBuffer(float[] output, float[] force, int nsamples) {
        for(int k=0;k<nsamples;k++) {
            output[k] = 0;
        }
        int nf = modalModel.nfUsed;
        for(int i=0;i<nf;i++) {
            float tmp_twoRCosTheta = twoRCosTheta[i];
            float tmp_R2 = R2[i];
            float tmp_a = ampR[i];
            float tmp_yt_1 = yt_1[i];
            float tmp_yt_2 = yt_2[i];
            for(int k=0;k<nsamples;k++) {
                float ynew = tmp_twoRCosTheta * tmp_yt_1 -
                    tmp_R2 * tmp_yt_2 + tmp_a * force[k];
                tmp_yt_2 = tmp_yt_1;
                tmp_yt_1 = ynew;
                output[k] += ynew;
            }
            yt_1[i] = tmp_yt_1;
            yt_2[i] = tmp_yt_2;
        }
    }

}
