import javax.sound.sampled.*;

/** Utility class for real-time audio play.
    Copyright The University of British Columbia, Canada
    @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class RTPlay {
    private float srate;
    private int bitsPerFrame;
    private int nchannels;
    private boolean signed;
    private static final boolean bigEndian = false;
    private AudioFormat audioFormat; 
    private SourceDataLine sourceDataLine;
    private static Mixer mixer;
    private static final int DEFAULT_BUFFERSIZE = 1024; //in bytes
    private int scratchBufsz; // Buffersize for writing audio
    private byte[] bbuf;

    static {
        Mixer.Info[] mixerinfo = AudioSystem.getMixerInfo();
        mixer = AudioSystem.getMixer(mixerinfo[0]);
    }
    
    private void initAudio(int buffersize, float srate,int bitsPerFrame,int nchannels,boolean signed) {
        this.srate = srate;
        this.bitsPerFrame = bitsPerFrame;
        this.nchannels = nchannels;
        this.signed = signed;
        audioFormat = new AudioFormat((float)srate,bitsPerFrame,nchannels,signed,bigEndian);
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);
                if (!mixer.isLineSupported(info)) {
            System.out.println(getClass().getName()+" : Error gettting sourcedataline: not supported\n");
        }       
        try {
            sourceDataLine = (SourceDataLine) mixer.getLine(info);
            if(buffersize == 0) {
                sourceDataLine.open(audioFormat);
            } else {
                sourceDataLine.open(audioFormat,buffersize);
            }
            sourceDataLine.start();
        } catch (LineUnavailableException ex) {
            System.out.println(getClass().getName()+" : Error getting line\n");
        }
        scratchBufsz = 4*2*sourceDataLine.getBufferSize();
        bbuf = new byte[scratchBufsz];
    }

    /** Constructor.
        @param bufferSize Buffer size used by JavaSound.
        @param srate sampling rate in Hertz.
        @param bitsPerFrame number of bits per audio frame.
        @param nchannels number of audio channels.
        @param signed true if signed format, false otherwise.
     */
    public RTPlay(int buffersize, float srate,int bitsPerFrame,int nchannels,boolean signed) {
        initAudio(buffersize,srate,bitsPerFrame,nchannels,signed);
    }

    /** Constructor. Uses default high latency JavaSound buffer size.
        @param srate sampling rate in Hertz.
        @param bitsPerFrame number of bits per audio frame.
        @param nchannels number of audio channels.
        @param signed true if signed format, false otherwise.
     */
    public RTPlay(float srate,int bitsPerFrame,int nchannels,boolean signed) {
        initAudio(0,srate,bitsPerFrame,nchannels,signed);
    }

    /** Write audio buffer to output queue and block if queue is full.
        @param y output buffer.
     */
    public void write(float [] y) {
        if(2 * y.length > scratchBufsz) {
            scratchBufsz = 2 * y.length;
            bbuf = new byte[scratchBufsz];
        }
        FormatUtils.floatToByte(bbuf,y);
        sourceDataLine.write(bbuf,0,2*y.length);
    }
    
}
